
-- SurgeonGeneral by Brimmstone
-- Version 1.2

SurgeonGeneral = LibStub("AceAddon-3.0"):NewAddon("SurgeonGeneral", "AceConsole-3.0", "AceEvent-3.0")

local options = {
	name = "SurgeonGeneral",
	handler = SurgeonGeneral,
    type = "group",
    args = {
		Reset = {
			type = "execute",
			name = "Reset",
			desc = "Reset Window Position",
			func = "ResetWindowPosition",
			order = 1,
		},
		Verbose = {
			type = "toggle",
			name = "Verbose",
			desc = "Toggles Verbose Mode",
			set = "ToggleVerbose",
			get = "GetVerbose",
			order = 3,
		},
		Condense = {
			type = "toggle",
			name = "Condense Assignments",
			desc = "Multiple healers with the same assignment are grouped",
			set = "ToggleCondense",
			get = "GetCondense",
			order = 2,
		},
		--[[LogToChat = {
			type = "toggle",
			name = "Log Assignments to Chat",
			desc = "Logs assignemnts when sent out via chat",
			set = "ToggleLogToChat",
			get = "GetLogToChat",
			order = 4,
		},]]--
	},
}

local defaults = {
	profile = {
		Verbose = false,
		Condense = true,
	},
}

local AssignmentDefaults = {
	global = {
		assignments = {},
	},
}

local Version = "r33"

local VariablesLoaded

local AutoFillInProgress
local AutoFillCurrentUnit
local AutoFillCurrentUnitClassE
local AutoFillCurrentUnitClassL
local AutoFillCurrentID
local AutoFillTaint
local AutoFillCurrentSlot

local RaidNameList = {"Karazhan","Gruul's Lair","Magtheridon's Lair","Serpentshrine Cavern","Tempest Keep","Mount Hyjal","Black Temple","Sunwell Plateau","Naxxramas","Obsidian Sanctum","Eye of Eternity"};
local BossNameList = {};
BossNameList.KZ = {"Attumen","Moroes","Maiden of Virtue","Opera Event","Nightbane","The Curator","Terestian Illhoof","Shade of Aran","Netherspite","Prince Malchezaar"}
BossNameList.GL = {"High King Maulgar","Gruul"}
BossNameList.ML = {"Magtheridon"}
BossNameList.SSC = {"Hydross the Unstable","The Lurker Below","Leotheras the Blind","Fathom Lord Karathress","Morogrim Tidewalker","Lasy Vashj"}
BossNameList.TK = {"Al'ar","Void Reaver","High Astromancer Solarian","Kael'thas Sunstrider"}
BossNameList.MH = {"Rage Winterchill","Anetheron","Kaz'rogal","Azgalor","Archimonde"}
BossNameList.BT = {"High Warlord Naj'entus","Supremus","Shade of Akama","Teron Gorefiend","Gurtogg Bloodboil","Reliquary of Souls","Mother Sharhaz","Illidari Council","Illidan Stormrage"};
BossNameList.SWP = {"Kalecgos","Brutallus","Felmyst","Eredar Twins","M'uru","Kil'jaeden"};
BossNameList.NAX = {"Anub'Rekhan","Grand Widow Faerlina","Maexxna","Noth the Plaguebringer","Heigan the Unclean","Loatheb","Instructor Razuvious","Gothic the Harvester","The Four Horsemen","Patchwerk","Grobbulus","Gluth","Thaddius","Sapphiron","Kel'Thuzad"};
BossNameList.OS = {"Shadron","Tenebron","Vesperon","Sartharion"};
BossNameList.EOE = {"Malygos"};

function SurgeonGeneral:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SurgeonGeneralDB", defaults, "Default")
	self.db2 = LibStub("AceDB-3.0"):New("SurgeonGeneralAssignments",AssignmentDefaults)
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SurgeonGeneral", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SurgeonGeneral", "SurgeonGeneral");
    self:RegisterChatCommand("sg", "ChatCommand");
	
	hooksecurefunc("NotifyInspect", SurgeonGeneral_NotifyInspect);
end

function SurgeonGeneral:ChatCommand(input)
	local x
	if not input or input:trim() == "" then
		SurgeonGeneral:ShowAssignmentFrame();
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SurgeonGeneral, "sg", "SurgeonGeneral", input)
	end
end

function SurgeonGeneral:ShowAssignmentFrame()
	SurgeonGeneralConfigFrame:Show();
	_G["VersionTextLabel"]:SetText(Version);
	SurgeonGeneral.db2.global.assignments["Clipboard"] = nil;
	UIDropDownMenu_SetWidth(RaidNameDropdown,175);
	UIDropDownMenu_SetWidth(BossNameDropdown,175);
	for x = 1,10 do
		if getglobal("HealerClass"..x.."Texture"):GetTexture() == nil then
			UpdateHealerClass(x);
			SurgeonGeneral:ArrangeClassDrags();
		end
	end
end

function SurgeonGeneral:ResetWindowPosition()
	SurgeonGeneral:ClearAllPoints();
	SurgeonGeneral:SetPoint("CENTER","UIParent");
end

function SurgeonGeneral:ToggleVerbose()
	self.db.profile.Verbose = not self.db.profile.Verbose
end

function SurgeonGeneral:GetVerbose()
	return self.db.profile.Verbose;
end

function SurgeonGeneral:ToggleCondense()
	self.db.profile.Condense = not self.db.profile.Condense
end

function SurgeonGeneral:GetCondense()
	return self.db.profile.Condense;
end

function SurgeonGeneral:ToggleLogToChat()
	self.db.profile.LogToChat = not self.db.profile.LogToChat
end

function SurgeonGeneral:GetLogToChat()
	return self.db.profile.LogToChat;
end

function SurgeonGeneral:CopyBossFromTarget()
	local name;
	if (UnitExists("target")) then
		if (UnitIsPlayer("target")) then
			return;
		else
			name,_ = UnitName("target");
			getglobal("BossName"):SetText(name);
		end
	end
end

function SurgeonGeneral:OnLoad()
	AutoFillInProgress = 0;
	VariablesLoaded = false;
	local x;
	getglobal("Phase1Name"):SetText("Phase1");
	getglobal("Phase2Name"):SetText("Phase2");
	getglobal("Phase3Name"):SetText("Phase3");
	getglobal("Phase4Name"):SetText("Phase4");
	getglobal("Phase5Name"):SetText("Phase5");
	SurgeonGeneral:RegisterEvent("INSPECT_TALENT_READY","AutoFillInspectReady");
	SurgeonGeneral:RegisterEvent("VARIABLES_LOADED","OnEvent");
	getglobal("AutoFillButton"):SetScript("OnClick",self.AutoFillStart);
	getglobal("ReportButton"):SetScript("OnClick",self.StartReport);
	getglobal("CopyAssignmentButton"):SetScript("OnClick",self.CopyAssignments);
	getglobal("PasteAssignmentButton"):SetScript("OnClick",self.PasteAssignments);
	getglobal("SaveAssignmentButton"):SetScript("OnClick",self.SaveAssignments);
	--getglobal("AutoFillBossButton"):SetScript("OnClick",self.CopyBossFromTarget);
end

function SurgeonGeneral:OnEvent(event)
	if(event == "VARIABLES_LOADED") then
		VariablesLoaded = true;
	end
end

function SetVar()
	--SurgeonGeneral:Print("Set Var");
end

function CalcPhases()
	local NumberOfPhases = 1;
	local k;
	k = 2;
	while k < 6 do
		if getglobal("Phase"..k.."Enable"):GetChecked() then
			NumberOfPhases = k;
			k = k + 1;
		else
			k = 6;
		end
	end
	if NumberOfPhases < 4 then
		Phase5Enable:Disable();
	else
		Phase5Enable:Enable();
	end
	
	if NumberOfPhases < 3 then
		Phase4Enable:Disable();
	else
		Phase4Enable:Enable();
	end
	
	if NumberOfPhases < 2 then
		Phase3Enable:Disable();
	else
		Phase3Enable:Enable();
	end
	
	return NumberOfPhases;
end

function PhaseNameChanged()
	-- placeholder
end

function ValidatePhaseNames()
	if (getglobal("Phase1Name"):GetText() == "") then
		getglobal("Phase1Name"):SetText("Phase1");
	end
	if (getglobal("Phase2Name"):GetText() == "") then
		getglobal("Phase2Name"):SetText("Phase2");
	end
	if (getglobal("Phase3Name"):GetText() == "") then
		getglobal("Phase3Name"):SetText("Phase3");
	end
	if (getglobal("Phase4Name"):GetText() == "") then
		getglobal("Phase4Name"):SetText("Phase4");
	end
	if (getglobal("Phase5Name"):GetText() == "") then
		getglobal("Phase5Name"):SetText("Phase5");
	end
end

function PhaseEnable_OnClick()
	CalcPhases();
end

function SurgeonGeneral:ReportToChannel(ReportChannel)
	local HealerMessage;
	local HealerClass;
	local j,k;
	local ReportChannelNumber;
	local ReportChannelName;
	
	ValidatePhaseNames();
	
	if (ReportChannel == "RAID") then
		ReportChannelNumber = nil;
		ReportChannelName = "RAID"
	else	
		ReportChannelNumber,_ = GetChannelName(ReportChannel);
		ReportChannelName = "CHANNEL"
		if(ReportChannelNumber == 0) then
			return;
		end
	end
	
	--SendChatMessage(" ",ReportChannelName,nil,ReportChannelNumber);
	if (UIDropDownMenu_GetText(BossNameDropdown) ~= nil) then
		SendChatMessage("Healing Assignments: "..UIDropDownMenu_GetText(BossNameDropdown),ReportChannelName,nil,ReportChannelNumber);
	else
		SendChatMessage("Healing Assignments",ReportChannelName,nil,ReportChannelNumber);
	end
	
	local AssignmentTable = {};	
	
	if CalcPhases() > 1 then
		HealerMessage = "Healer:  ";
		HealerMessage = HealerMessage .. getglobal("Phase1Name"):GetText();
		for j = 2,CalcPhases() do
			HealerMessage = HealerMessage .. " ~ " .. getglobal("Phase"..j.."Name"):GetText();
		end
		SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
	end
	
	for k = 1,10 do
		if not (getglobal("HealerName"..k):GetText() == "") then
			HealerMessage = ""
			HealerMessage = HealerMessage .. getglobal("Healer"..k.."Phase1"):GetText();
			for j = 2,CalcPhases() do
				HealerMessage = HealerMessage .. " ~ " .. getglobal("Healer"..k.."Phase"..j):GetText();
			end
						
			if (not self.db.profile.Condense) then
				HealerMessage = getglobal("HealerName"..k):GetText() .. ":  "..HealerMessage;
				SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
			else
				if AssignmentTable[HealerMessage] == nil then
					AssignmentTable[HealerMessage] = getglobal("HealerName"..k):GetText();
				else
					AssignmentTable[HealerMessage] = AssignmentTable[HealerMessage]..", "..getglobal("HealerName"..k):GetText();
				end
			end
		end
	end
	
	if (not self.db.profile.Condense) then
		return;
	end
	
	for index,value in pairs(AssignmentTable) do
		if (type(index) == "number") then
			index = tostring(index)
		elseif (type(index) == "boolean") then
			if (index) then
				index = "true"
			else
				index = "false"
			end
		end
		SendChatMessage(value..": "..index,ReportChannelName,nil,ReportChannelNumber);
	end
end

function SurgeonGeneral:StartReport()
	local AssignmentsLogged
	
	if getglobal("RaidChannelEnable"):GetChecked() then
		SurgeonGeneral:ReportToChannel("RAID");
	end
	
	if getglobal("CustomChannelEnable"):GetChecked() then
		SurgeonGeneral:ReportToChannel(getglobal("CustomChannelName"):GetText());
	end
end

function SurgeonGeneral:VerbosePrint(msg,color)
	if (self.db.profile.Verbose) then
		if color ~= nil then
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cff"..color..msg.."|r");
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cffffffff"..msg.."|r");
		end
	end
end

function SurgeonGeneral:ForcePrint(msg,color)
	if color ~= nil then
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cff"..color..msg.."|r");
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cffffffff"..msg.."|r");
	end
end

function SurgeonGeneral:AutoFillStart()
	if (not UnitInRaid("player")) then
		SurgeonGeneral:ForcePrint("You are not in a raid","ffff00");
		AutoFillInProgress = 0;
		return;
	end
	
	SurgeonGeneral:VerbosePrint("Auto Fill Starting...");
	
	AutoFillInProgress = 1;
	AutoFillCurrentID = 0;
	AutoFillCurrentSlot = 1;
	
	SurgeonGeneral:AutoFillNext();	
end

function SurgeonGeneral:AutoFillNext()
	if (AutoFillInProgress == 1) then
		while (AutoFillCurrentID < GetNumRaidMembers()) do
		
			-- break function if Autofill terminated while in this loop.
			if (AutoFillInProgress == 0) then
				return;
			end
			
			AutoFillCurrentID = AutoFillCurrentID + 1;
			AutoFillCurrentUnit = "raid"..AutoFillCurrentID;
			AutoFillCurrentUnitClassL,AutoFillCurrentUnitClassE = UnitClass(AutoFillCurrentUnit);			
			if ((AutoFillCurrentUnitClassE == "WARRIOR") or (AutoFillCurrentUnitClassE == "ROGUE") or (AutoFillCurrentUnitClassE == "HUNTER") or (AutoFillCurrentUnitClassE == "MAGE") or (AutoFillCurrentUnitClassE == "WARLOCK")) then
				-- If it's not a healing class, go to next unit
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is not a healing class.");
			elseif UnitIsUnit("player",AutoFillCurrentUnit) then
				-- If it's me, just call the function to get my info (You can't inspect yourself).
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is me.");
				SurgeonGeneral:AutoFillGetTalents();
			elseif (CheckInteractDistance(AutoFillCurrentUnit,1)) then
				-- If it's somebody else and they're within inspect range, get their info.
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is a "..AutoFillCurrentUnitClassL..".  Inspecting...");
				NotifyInspect(AutoFillCurrentUnit);
				return;
			else
				-- If they're not within range, just skip them and move along
				SurgeonGeneral:ForcePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." was out of inspection range.");
			end
		end
		if (AutoFillInProgress == 1) then
			SurgeonGeneral:AutoFillStop();
		end
	end
end

function SurgeonGeneral:AutoFillStop()
	AutoFillInProgress = 0;
	SurgeonGeneral:ForcePrint("Auto Fill Complete");
end

function SurgeonGeneral_NotifyInspect(unit)
	if (AutoFillInProgress == 0) then
		return;
	elseif (unit == AutoFillCurrentUnit) then
		AutoFillTaint = 0;
	else
		AutoFillTaint = 1;
		SurgeonGeneral:VerbosePrint("Inspect Taint!","ff0000");
		NotifyInspect(AutoFillCurrentUnit);
	end
end

function SurgeonGeneral:AutoFillGetTalents()
	local HealingTalents;
	local HealingTree;
	local HealingTreeName;
	local InspectFlag;
	local TalentThreshold = 45;
	
	if (UnitIsUnit("player",AutoFillCurrentUnit)) then
		InspectFlag = false;
	else
		InspectFlag = true;
	end
		
	if (AutoFillInProgress == 0) then
		return;
	elseif (AutoFillTaint == 1) then
		return;
	end
	
	-- Get number of healing talents/tree name.
	if (AutoFillCurrentUnitClassE == "PALADIN") then
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(1,InspectFlag);
	elseif (AutoFillCurrentUnitClassE == "PRIEST") then
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(2,InspectFlag);
		-- if Holy tree doesn't meet threshold, check for Disc healer.
		-- Thanks for P5hema2 from Curse for their user feedback.
		if HealingTalents < TalentThreshold then
			HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(1,InspectFlag);
		end
	elseif (AutoFillCurrentUnitClassE == "SHAMAN") then
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(3,InspectFlag);
	elseif (AutoFillCurrentUnitClassE == "DRUID") then
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(3,InspectFlag);
	else
		--  Not a healing class... should not have been inspected, but trapping anyways.
		return;
	end
	
	-- Assuming if you call yourself a healer, you've put at least 36 talents in your healing tree.
	SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." has "..HealingTalents.." in the "..HealingTreeName.." tree.");
	if (HealingTalents >= TalentThreshold) then
		local HealerName,_ = UnitName(AutoFillCurrentUnit);
		getglobal("HealerName"..AutoFillCurrentSlot):SetText(HealerName);
		AutoFillCurrentSlot = AutoFillCurrentSlot + 1;
	end
	
	if ((AutoFillCurrentSlot < 11) and (AutoFillCurrentID < GetNumRaidMembers())) then
		SurgeonGeneral:AutoFillNext();
	else
		SurgeonGeneral:AutoFillStop();
	end
end

function SurgeonGeneral:AutoFillInspectReady()
	SurgeonGeneral:AutoFillGetTalents();
end

function HealerNameChange(healer)
	local id = tonumber(string.sub(healer:GetName(),11))
	UpdateHealerClass(id);
end

function UpdateHealerClass(icon)
	local _,englishClass = UnitClass(getglobal("HealerName"..icon):GetText());
	if englishClass ~= nil then
		getglobal("HealerClass"..icon.."Texture"):SetTexture("Interface\\Addons\\SurgeonGeneral\\Icons\\"..englishClass);
		getglobal("HealerClass"..icon.."DragTexture"):SetTexture("Interface\\Addons\\SurgeonGeneral\\Icons\\"..englishClass);
	else
		getglobal("HealerClass"..icon.."Texture"):SetTexture(nil);
		getglobal("HealerClass"..icon.."DragTexture"):SetTexture(nil);
	end
end

function EditBox_OnEscapePressed()
	SurgeonGeneralConfigFrame:Hide()
end

function SurgeonGeneral:SaveToFile(Location)
	if type(SurgeonGeneral.db2.global.assignments) == nil then
		SurgeonGeneral.db2.global.assignments = {};
	end
	SurgeonGeneral.db2.global.assignments[Location] = 
	{getglobal("Phase1Name"):GetText(), getglobal("Phase2Name"):GetText(),getglobal("Phase3Name"):GetText(),getglobal("Phase4Name"):GetText(),getglobal("Phase5Name"):GetText(),
	 getglobal("Healer1Phase1"):GetText(),getglobal("Healer1Phase2"):GetText(),getglobal("Healer1Phase3"):GetText(),getglobal("Healer1Phase4"):GetText(),getglobal("Healer1Phase5"):GetText(),
	 getglobal("Healer2Phase1"):GetText(),getglobal("Healer2Phase2"):GetText(),getglobal("Healer2Phase3"):GetText(),getglobal("Healer2Phase4"):GetText(),getglobal("Healer2Phase5"):GetText(),
	 getglobal("Healer3Phase1"):GetText(),getglobal("Healer3Phase2"):GetText(),getglobal("Healer3Phase3"):GetText(),getglobal("Healer3Phase4"):GetText(),getglobal("Healer3Phase5"):GetText(),
	 getglobal("Healer4Phase1"):GetText(),getglobal("Healer4Phase2"):GetText(),getglobal("Healer4Phase3"):GetText(),getglobal("Healer4Phase4"):GetText(),getglobal("Healer4Phase5"):GetText(),
	 getglobal("Healer5Phase1"):GetText(),getglobal("Healer5Phase2"):GetText(),getglobal("Healer5Phase3"):GetText(),getglobal("Healer5Phase4"):GetText(),getglobal("Healer5Phase5"):GetText(),
	 getglobal("Healer6Phase1"):GetText(),getglobal("Healer6Phase2"):GetText(),getglobal("Healer6Phase3"):GetText(),getglobal("Healer6Phase4"):GetText(),getglobal("Healer6Phase5"):GetText(),
	 getglobal("Healer7Phase1"):GetText(),getglobal("Healer7Phase2"):GetText(),getglobal("Healer7Phase3"):GetText(),getglobal("Healer7Phase4"):GetText(),getglobal("Healer7Phase5"):GetText(),
	 getglobal("Healer8Phase1"):GetText(),getglobal("Healer8Phase2"):GetText(),getglobal("Healer8Phase3"):GetText(),getglobal("Healer8Phase4"):GetText(),getglobal("Healer8Phase5"):GetText(),
	 getglobal("Healer9Phase1"):GetText(),getglobal("Healer9Phase2"):GetText(),getglobal("Healer9Phase3"):GetText(),getglobal("Healer9Phase4"):GetText(),getglobal("Healer9Phase5"):GetText(),
	 getglobal("Healer10Phase1"):GetText(),getglobal("Healer10Phase2"):GetText(),getglobal("Healer10Phase3"):GetText(),getglobal("Healer10Phase4"):GetText(),getglobal("Healer10Phase5"):GetText(),
	 getglobal("Phase2Enable"):GetChecked(),getglobal("Phase3Enable"):GetChecked(),getglobal("Phase4Enable"):GetChecked(),getglobal("Phase5Enable"):GetChecked()}
end

function SurgeonGeneral:LoadFromFile(Location)
	local x,y,z;
	if SurgeonGeneral.db2.global.assignments[Location] ~= nil then
		for x=1,5 do
			getglobal("Phase"..x.."Name"):SetText(SurgeonGeneral.db2.global.assignments[Location][x]);
		end
		
		for x=1,10 do
			for y=1,5 do
				z = (x-1)*5+y+5;
				getglobal("Healer"..x.."Phase"..y):SetText(SurgeonGeneral.db2.global.assignments[Location][z]);
			end
		end
		
		for x=2,5 do
			getglobal("Phase"..x.."Enable"):SetChecked(SurgeonGeneral.db2.global.assignments[Location][x+54]);
		end
		CalcPhases();
	else
		SurgeonGeneral:ForcePrint("No data saved to "..Location)
	end
end



function SurgeonGeneral:SaveAssignments()
	SurgeonGeneral:SaveToFile(UIDropDownMenu_GetText(BossNameDropdown));
end

function SurgeonGeneral:RecallAssignments(BossName)
	SurgeonGeneral:LoadFromFile(UIDropDownMenu_GetText(BossNameDropdown));
end

function SurgeonGeneral:CopyAssignments()
	SurgeonGeneral:SaveToFile("Clipboard");
end

function SurgeonGeneral:PasteAssignments()
	SurgeonGeneral:LoadFromFile("Clipboard");
end


local info = {};

function RaidNameDropdown_OnLoad()
	if (VariablesLoaded == false) then
		return;
	end;
	
	local x;
	local List = RaidNameList;
	for x=1,getn(List) do
		info.text = List[x];
		info.value = x;
		info.owner = this:GetParent();
		info.func = function() RaidName_OnSelect() end;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
end

function RaidName_OnSelect()
	if (UIDropDownMenu_GetSelectedValue(this.owner) ~= this.value) then
		UIDropDownMenu_SetSelectedValue(this.owner, this.value);
		UIDropDownMenu_ClearAll(BossNameDropdown);
		BossNameDropdown_OnLoad();
	end
end

function BossNameDropdown_OnLoad()
	if (VariablesLoaded == false) then
		return;
	end;
	
	local x;
	local List = {};
	
	if UIDropDownMenu_GetSelectedValue(RaidNameDropdown) ~= nil then
		List = select(UIDropDownMenu_GetSelectedValue(RaidNameDropdown),BossNameList.KZ, BossNameList.GL, BossNameList.ML, BossNameList.SSC, BossNameList.TK, BossNameList.MH, BossNameList.BT, BossNameList.SWP, BossNameList.NAX, BossNameList.OS, BossNameList.EOE);
	end

	for x=1,getn(List) do
		info.text = List[x];
		info.value = x;
		info.owner = this:GetParent();
		info.hasarrow = true;
		info.func = function() BossName_OnSelect() end;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
	
	if(UIDropDownMenu_GetSelectedValue(BossNameDropdown) == nil) then
		UIDropDownMenu_SetSelectedValue(BossNameDropdown,1)
		UIDropDownMenu_SetText(BossNameDropdown,List[1])
		SurgeonGeneral:LoadFromFile(UIDropDownMenu_GetText(BossNameDropdown));
	end
end

function BossName_OnSelect()
	UIDropDownMenu_SetSelectedValue(this.owner, this.value);
	SurgeonGeneral:LoadFromFile(UIDropDownMenu_GetText(BossNameDropdown));
end


local LDB = LibStub("LibDataBroker-1.1", true)
if LDB then
	local L_SG_LEFT = "Left click to open setup window"
	local L_SG_RIGHT = "Right click to report assignments"

	LibStub("LibDataBroker-1.1"):NewDataObject("SG", {
		type = "launcher",
		text = "SG",
		OnClick = function(_, msg)
			if msg == "LeftButton" then
				if _G["SurgeonGeneralConfigFrame"]:IsShown() then
					_G["SurgeonGeneralConfigFrame"]:Hide()
				else
					SurgeonGeneral:ShowAssignmentFrame();
				end
			elseif msg == "RightButton" then
				SurgeonGeneral:StartReport();
			end
		end,
		icon = "Interface\\Icons\\Spell_ChargePositive",
		OnTooltipShow = function(tooltip)
			if not tooltip or not tooltip.AddLine then return end
			tooltip:AddLine("SurgeonGeneral")
			tooltip:AddLine(L_SG_LEFT)
			tooltip:AddLine(L_SG_RIGHT)
		end,
	})
end

function SurgeonGeneral:ArrangeClassDrags()
	local x
	
	for x=1,10 do
		_G["HealerClass"..x.."Drag"]:SetAlpha(0.0);
		_G["HealerClass"..x.."Drag"]:ClearAllPoints();
		_G["HealerClass"..x.."Drag"]:SetPoint("CENTER","HealerClass"..x);
	end
end

function SurgeonGeneral:HealerSwap(x,y)
	local t;
	t = _G["HealerName"..y]:GetText();
	_G["HealerName"..y]:SetText(_G["HealerName"..x]:GetText())
	_G["HealerName"..x]:SetText(t)
end

function SurgeonGeneral:IconOnClick()
	--SurgeonGeneral:ForcePrint("On Click");
end

function SurgeonGeneral:IconDragStart(frame,...)
	local id = tonumber(string.sub(frame:GetName(),12,-5))
	
	_G["HealerClass"..id.."Drag"]:SetAlpha(0.75);
	_G["HealerClass"..id.."Drag"]:SetFrameLevel(_G["HealerClass"..id]:GetFrameLevel()+1)
end

function SurgeonGeneral:IconDragStop(frame,...)
	local id = tonumber(string.sub(frame:GetName(),12,-5))
	local x;
	
	SurgeonGeneral:ArrangeClassDrags();
	
	if _G["HealerClass"..id.."Texture"]:GetTexture() == nil then
		return;
	end
	
	for x=1,10 do
		if MouseIsOver(_G["HealerClass"..x]) then
			SurgeonGeneral:HealerSwap(id,x);
		end
	end
end